@@ -28,7 +28,7 @@ struct ipkg_cmd
 };
 typedef struct ipkg_cmd ipkg_cmd_t;
 
-ipkg_cmd_t *ipkg_cmd_find(char *name);
+ipkg_cmd_t *ipkg_cmd_find(const char *name);
 int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, char *argv[]);
 int ipkg_multiple_files_scan (ipkg_conf_t *conf, int argc, char *argv[]);
 /* install any packges with state_want == SW_INSTALL */
