@@ -43,25 +43,26 @@ static int set_and_load_pkg_dest_list(ipkg_conf_t *conf,
 void ipkg_init_options_array(const ipkg_conf_t *conf, ipkg_option_t **options)
 {
      ipkg_option_t tmp[] = {
-	  { "force_depends", IPKG_OPT_TYPE_BOOL, &conf->force_depends },
 	  { "force_defaults", IPKG_OPT_TYPE_BOOL, &conf->force_defaults },
+	  { "force_depends", IPKG_OPT_TYPE_BOOL, &conf->force_depends },
 	  { "force_overwrite", IPKG_OPT_TYPE_BOOL, &conf->force_overwrite },
 	  { "force_reinstall", IPKG_OPT_TYPE_BOOL, &conf->force_reinstall },
 	  { "force_space", IPKG_OPT_TYPE_BOOL, &conf->force_space },
-	  { "nodeps", IPKG_OPT_TYPE_BOOL, &conf->nodeps },
-	  { "verbose_wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
-	  { "verbose-wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
-	  { "verbosity", IPKG_OPT_TYPE_INT, &conf->verbosity },
-	  { "offline_root", IPKG_OPT_TYPE_STRING, &conf->offline_root },
-	  { "offline_root_pre_script_cmd", IPKG_OPT_TYPE_STRING, &conf->offline_root_pre_script_cmd },
-	  { "offline_root_post_script_cmd", IPKG_OPT_TYPE_STRING, &conf->offline_root_post_script_cmd },
-	  { "query-all", IPKG_OPT_TYPE_BOOL, &conf->query_all },
-	  { "http_proxy", IPKG_OPT_TYPE_STRING, &conf->http_proxy },
 	  { "ftp_proxy", IPKG_OPT_TYPE_STRING, &conf->ftp_proxy },
-	  { "no_proxy", IPKG_OPT_TYPE_STRING, &conf->no_proxy },
-	  { "proxy_user", IPKG_OPT_TYPE_STRING, &conf->proxy_user },
-	  { "proxy_passwd", IPKG_OPT_TYPE_STRING, &conf->proxy_passwd },
+	  { "http_proxy", IPKG_OPT_TYPE_STRING, &conf->http_proxy },
 	  { "multiple_providers", IPKG_OPT_TYPE_BOOL, &conf->multiple_providers },
+	  { "no_proxy", IPKG_OPT_TYPE_STRING, &conf->no_proxy },
+	  { "test", IPKG_OPT_TYPE_INT, &conf->noaction },
+	  { "noaction", IPKG_OPT_TYPE_INT, &conf->noaction },
+	  { "nodeps", IPKG_OPT_TYPE_BOOL, &conf->nodeps },
+	  { "offline_root", IPKG_OPT_TYPE_STRING, &conf->offline_root },
+	  { "offline_root_post_script_cmd", IPKG_OPT_TYPE_STRING, &conf->offline_root_post_script_cmd },
+	  { "offline_root_pre_script_cmd", IPKG_OPT_TYPE_STRING, &conf->offline_root_pre_script_cmd },
+	  { "proxy_passwd", IPKG_OPT_TYPE_STRING, &conf->proxy_passwd },
+	  { "proxy_user", IPKG_OPT_TYPE_STRING, &conf->proxy_user },
+	  { "query-all", IPKG_OPT_TYPE_BOOL, &conf->query_all },
+	  { "verbose-wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
+	  { "verbose_wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
 	  { NULL }
      };
 
@@ -128,6 +129,7 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      conf->force_defaults = 0;
      conf->force_overwrite = 0;
      conf->force_reinstall = 0;
+     conf->force_space = 0;
      conf->force_removal_of_essential_packages = 0;
      conf->force_removal_of_dependent_packages = 0;
      conf->nodeps = 0;
@@ -137,6 +139,7 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      conf->offline_root_post_script_cmd = NULL;
      conf->multiple_providers = 0;
      conf->verbosity = 0;
+     conf->noaction = 0;
 
      conf->http_proxy = NULL;
      conf->ftp_proxy = NULL;
@@ -211,6 +214,9 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      if (args->nodeps) {
 	  conf->nodeps = 1;
      }
+     if (args->noaction) {
+	  conf->noaction = 1;
+     }
      if (args->query_all) {
 	  conf->query_all = 1;
      }
@@ -264,11 +270,13 @@ void ipkg_conf_deinit(ipkg_conf_t *conf)
      err = rmdir(conf->tmp_dir);
      if (err) {
 	  if (errno == ENOTEMPTY) {
-	       fprintf(stderr, "WARNING: Temporary directory %s was not "
-		       "cleaned up properly.\n\tKindly complain to the "
-		       "maintainer of ipkg\n", conf->tmp_dir);
+	       char *cmd;
+	       sprintf_alloc(&cmd, "rm -fr %s\n", conf->tmp_dir);
+	       err = xsystem(cmd);
+	       free(cmd);
 	  }
-	  fprintf(stderr, "WARNING: Unable to remove temporary directory: %s: %s\n", conf->tmp_dir, strerror(errno));
+	  if (err)
+	       fprintf(stderr, "WARNING: Unable to remove temporary directory: %s: %s\n", conf->tmp_dir, strerror(errno));
      }
 #endif /* IPKG_DEBUG_NO_TMP_CLEANUP */
 
@@ -302,7 +310,7 @@ void ipkg_conf_deinit(ipkg_conf_t *conf)
 		    if (len > c) 
 			 c = len;
 	       }
-	       ipkg_message(conf, IPKG_NOTICE, "hash_table[%s] n_buckets=%d n_elements=%d max_conflicts=%d n_conflicts=%d\n", 
+	       ipkg_message(conf, IPKG_DEBUG, "hash_table[%s] n_buckets=%d n_elements=%d max_conflicts=%d n_conflicts=%d\n", 
 			    hash->name, hash->n_entries, hash->n_elements, c, n_conflicts);
 	       hash_table_deinit(hash);
 	  }
@@ -484,9 +492,19 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 	  if (strcmp(type, "option") == 0) {
 	       ipkg_conf_set_option(options, name, value);
 	  } else if (strcmp(type, "src") == 0) {
-	       pkg_src_list_append (pkg_src_list, name, value, extra, 0);
+	       if (!nv_pair_list_find(pkg_src_list, name)) {
+		    pkg_src_list_append (pkg_src_list, name, value, extra, 0);
+	       } else {
+		    ipkg_message(conf, IPKG_ERROR, "ERROR: duplicate src declaration.  Skipping:\n\t src %s %s\n",
+				 name, value);
+	       }
 	  } else if (strcmp(type, "src/gz") == 0) {
-	       pkg_src_list_append (pkg_src_list, name, value, extra, 1);
+	       if (!nv_pair_list_find(pkg_src_list, name)) {
+		    pkg_src_list_append (pkg_src_list, name, value, extra, 1);
+	       } else {
+		    ipkg_message(conf, IPKG_ERROR, "ERROR: duplicate src declaration.  Skipping:\n\t src %s %s\n",
+				 name, value);
+	       }
 	  } else if (strcmp(type, "dest") == 0) {
 	       nv_pair_list_append(tmp_dest_nv_pair_list, name, value);
 	  } else if (strcmp(type, "arch") == 0) {
@@ -566,6 +584,8 @@ int ipkg_conf_write_status_files(ipkg_conf_t *conf)
      register int i;
      int err;
 
+     if (conf->noaction)
+	  return 0;
      for (iter = conf->pkg_dest_list.head; iter; iter = iter->next) {
 	  dest = iter->data;
 	  dest->status_file = fopen(dest->status_file_tmp_name, "w");
